#!/usr/bin/env python

# Some constants I've found deep in the sources of xosd.
# Vertical anchors:
XOSD_TOP    = 0     # Align the OSD to the top of the screen.
XOSD_BOTTOM = 1     # Align the OSD to the bottom of the screen
XOSD_MIDDLE = 2     # Align the OSD to the middle of the screen
# Horizontal anchors:
XOSD_LEFT   = 0     # Align the OSD to the left side of the screen.
XOSD_CENTER = 1     # Align the OSD to the center of the screen.
XOSD_RIGHT  = 2     # Align the OSD to the right side of the screen.


# You can customize the OSD here.
# (You may retrieve those font strings by using the program xfontsel)
FONT            = '-microsoft-verdana-bold-r-normal-*-*-90-*-*-p-*-viscii1.1-1'
FONT_COLOR      = "#FFFFFF"
H_ALIGN         = XOSD_RIGHT # The horizontal anchor
V_ALIGN         = XOSD_TOP   # The vertical anchor
H_OFFSET        = 100         # The space in pixels between the OSD and the horizontal anchor
V_OFFSET        = 3          # The space in pixels between the OSD and the vertical anchor
SHADOW_COLOR    = "#000000"  # The color of the text shadow
SHADOW_OFFSET   = 1          # The offset of the text shadow in pixels
UPDATE_INTERVAL = 1          # The time in seconds before the OSD is updated

from os     import popen
from time   import sleep

try:
    from pydcop import anyAppCalled
except ImportError:
    os.popen( "kdialog --sorry 'Pydcop (DCOP bindings for Python) is required to run this plugin.'" )
    raise

try:
    import pyosd
except ImportError:
    os.popen( "kdialog --sorry 'Pyosd (XOSD bindings for Python) is required to run this plugin.'" )
    raise

# Returns the song information in this format:
# <artist> - <title> (-<remaining time>)
def getSongInfos():
    # Get artist and title.
    nowPlaying = anyAppCalled("amarok").player.nowPlaying()
    
    # Get the remaining time only if a song is played currently.
    if nowPlaying != "":
        trackLength   = anyAppCalled("amarok").player.trackTotalTime()
        secondsPlayed = anyAppCalled("amarok").player.trackCurrentTime()
        remainingTime = int(trackLength) - int(secondsPlayed)
        minutes, seconds = divmod(remainingTime, 60) 
        nowPlaying = nowPlaying + " (-" + str(minutes) + ":" + '%(#)02d' % {"#": seconds} + ")"
    return nowPlaying 

def main():
    try:
        # Initialize the XOSD object. There may be some redundancies here, but 
        # I found no good documentation of the python XOSD bindings and this 
        # way it does work at all :-)
        osd = pyosd.osd(FONT, FONT_COLOR, pos=0, shadow=0, timeout=(UPDATE_INTERVAL + 1), offset=60)
        osd.set_font(FONT)
        osd.set_colour(FONT_COLOR)
        osd.set_align(H_ALIGN)
        osd.set_pos(V_ALIGN)
        osd.set_horizontal_offset(H_OFFSET)
        osd.set_vertical_offset(V_OFFSET)
        osd.set_shadow_offset(SHADOW_OFFSET)
        osd.set_shadow_colour(SHADOW_COLOR)
    except pyosd.error, e:
        print("XOSD Initialization failed: " + str(e))
        return

    # Refresh the OSD each UPDATE_INTERVAL until the script is stopped
    while True:
	# Convert the string from utf-8 to latin1 to display some special 
	# characters like german umlauts correctly. This must be done because
	# amarok returns an utf-8 string whereas XOSD needs latin1 encoded strings
	output = unicode(getSongInfos(), "utf-8"); # utf-8   -> unicode
        osd.display(output.encode("latin1"))       # unicode -> latin1
        sleep(UPDATE_INTERVAL)


# Called when script is stopped by user
def onStop(signum, stackframe):
    pass

if __name__ == "__main__":
    main()
